package cz.cvut.kbss.bookstore.dao;

import java.util.List;

/**
 * Defines methods provided by all data access objects.
 *
 * @param <T>  Entity type
 * @param <ID> Identifier type
 */
public interface GenericDao<T, ID> {

    /**
     * Gets all instances, ordered by their id.
     *
     * @return List of entities of the type managed by this DAO
     */
    public List<T> findAll();

    /**
     * Finds entity by the specified ID.
     *
     * @param id ID to search by
     * @return Matching instance or null if there is no such
     */
    public T find(ID id);

    /**
     * Gets the number of instances of the type managed by this DAO.
     *
     * @return Instance count
     */
    public Long getCount();

    /**
     * Removes the specified instance
     *
     * @param instance Instance to remove
     */
    public void remove(T instance);

    /**
     * Persists the specified instance.
     *
     * @param instance Instance to persist
     */
    public void persist(T instance);

    /**
     * Merges state of the specified instance into the database.
     *
     * @param instance Instance with updated data
     */
    public void update(T instance);

    /**
     * Loads just a reference (proxy) of instance with the specified ID.
     * <p>
     * Useful e.g. for remove, where the actual state of the instance is not important.
     *
     * @param id Instance id
     * @return Instance reference or null, if there is no matching instance
     */
    public T loadById(ID id);
}
